@@ -4,17 +4,29 @@ Inference utilities.
 from enum import auto, Enum
 from typing import Any, Callable, List, Optional, Sequence, Tuple
 
-import cv2  # type: ignore
+import cv2
 import numpy as np
 from PIL import Image  # type: ignore
 from tqdm import tqdm  # type: ignore
 
+from keras import KerasTensor, Model  # type: ignore
+
 from ._download import get_default_weights_path
 from ._image import preprocess_image, Preprocessing
-from ._inspection import make_and_save_nsfw_grad_cam
 from ._model import make_open_nsfw_model
 from ._typing import NDFloat32Array
 
+global_model: Optional[Model] = None
+global_model_path: Optional[str] = None
+
+
+def _update_global_model_if_needed(weights_path: Optional[str]) -> None:
+    global global_model
+    global global_model_path
+    if global_model is None or weights_path != global_model_path:
+        global_model = make_open_nsfw_model(weights_path=weights_path)
+        global_model_path = weights_path
+
 
 def predict_image(
         image_path: str,
@@ -29,19 +41,44 @@ def predict_image(
     """
     pil_image = Image.open(image_path)
     image = preprocess_image(pil_image, preprocessing)
-    model = make_open_nsfw_model(weights_path=weights_path)
-    nsfw_probability = float(
-        model.predict(np.expand_dims(image, 0), batch_size=1, verbose=0)[0][1]
-    )
+    _update_global_model_if_needed(weights_path)
+    assert global_model is not None
+    nsfw_probability = float(global_model(np.expand_dims(image, 0))[0][1])
 
     if grad_cam_path is not None:
+        # TensorFlow will only be imported here.
+        from ._inspection import make_and_save_nsfw_grad_cam
+
         make_and_save_nsfw_grad_cam(
-            pil_image, preprocessing, model, grad_cam_path, alpha
+            pil_image, preprocessing, global_model, grad_cam_path, alpha
         )
 
     return nsfw_probability
 
 
+def _predict_from_image_paths_in_batches(
+        model_: Model,
+        image_paths: Sequence[str],
+        batch_size: int,
+        preprocessing: Preprocessing
+) -> NDFloat32Array:
+    """
+    It is on purpose not to use `model.predict` because many users would like
+    to use the API in loops. See here:
+    https://keras.io/api/models/model_training_apis/#predict-method
+    """
+    prediction_batches: List[KerasTensor] = []
+    for i in range(0, len(image_paths), batch_size):
+        path_batch = image_paths[i: i + batch_size]
+        image_batch = [
+            preprocess_image(Image.open(path), preprocessing)
+            for path in path_batch
+        ]
+        prediction_batches.append(model_(np.array(image_batch)))
+    predictions: NDFloat32Array = np.concatenate(prediction_batches, axis=0)
+    return predictions
+
+
 def predict_images(
         image_paths: Sequence[str],
         batch_size: int = 8,
@@ -54,18 +91,19 @@ def predict_images(
     Pipeline from image paths to predicted NSFW probabilities.
     Optionally generate and save the Grad-CAM plots.
     """
-    images = np.array([
-        preprocess_image(Image.open(image_path), preprocessing)
-        for image_path in image_paths
-    ])
-    model = make_open_nsfw_model(weights_path=weights_path)
-    predictions = model.predict(images, batch_size=batch_size, verbose=0)
+    _update_global_model_if_needed(weights_path)
+    predictions = _predict_from_image_paths_in_batches(
+        global_model, image_paths, batch_size, preprocessing
+    )
     nsfw_probabilities: List[float] = predictions[:, 1].tolist()
 
     if grad_cam_paths is not None:
+        # TensorFlow will only be imported here.
+        from ._inspection import make_and_save_nsfw_grad_cam
+
         for image_path, grad_cam_path in zip(image_paths, grad_cam_paths):
             make_and_save_nsfw_grad_cam(
-                Image.open(image_path), preprocessing, model,
+                Image.open(image_path), preprocessing, global_model,
                 grad_cam_path, alpha
             )
 
@@ -95,6 +133,19 @@ def _get_aggregation_fn(
     return fn
 
 
+def _predict_preprocessed_images_in_batches(
+        model_: Model,
+        images: List[NDFloat32Array],
+        batch_size: int
+) -> NDFloat32Array:
+    prediction_batches: List[KerasTensor] = []
+    for i in range(0, len(images), batch_size):
+        batch = np.array(images[i: i + batch_size])
+        prediction_batches.append(model_(batch))
+    predictions: NDFloat32Array = np.concatenate(prediction_batches, axis=0)
+    return predictions
+
+
 def predict_video_frames(
         video_path: str,
         frame_interval: int = 8,
@@ -112,7 +163,7 @@ def predict_video_frames(
     cap = cv2.VideoCapture(video_path)  # pylint: disable=no-member
     fps = cap.get(cv2.CAP_PROP_FPS)  # pylint: disable=no-member
 
-    model = make_open_nsfw_model(weights_path=weights_path)
+    _update_global_model_if_needed(weights_path)
 
     video_writer: Optional[cv2.VideoWriter] = None  # pylint: disable=no-member
     input_frames: List[NDFloat32Array] = []
@@ -139,7 +190,7 @@ def predict_video_frames(
         if video_writer is None and output_video_path is not None:
             video_writer = cv2.VideoWriter(  # pylint: disable=no-member
                 output_video_path,
-                cv2.VideoWriter_fourcc(*"mp4v"),  # pylint: disable=no-member
+                cv2.VideoWriter_fourcc(*"mp4v"),  # type: ignore # pylint: disable=no-member
                 fps, (frame.shape[1], frame.shape[0])
             )
 
@@ -149,8 +200,8 @@ def predict_video_frames(
             input_frames.append(input_frame)
 
             if frame_count == 1 or len(input_frames) >= aggregation_size:
-                predictions = model.predict(
-                    np.array(input_frames), batch_size=batch_size, verbose=0
+                predictions = _predict_preprocessed_images_in_batches(
+                    global_model, input_frames, batch_size
                 )
                 agg_fn = _get_aggregation_fn(aggregation)
                 nsfw_probability = agg_fn(predictions[:, 1])
