@@ -6,13 +6,15 @@ https://github.com/mdietrichstein/tensorflow-open_nsfw
 https://github.com/yahoo/open_nsfw
 """
 import os
-from typing import Optional, Tuple
+from typing import Optional, Tuple, TypeVar
 
-import tensorflow as tf  # type: ignore
-from tensorflow.keras import layers  # type: ignore # pylint: disable=import-error
+from keras import layers, Model  # type: ignore
 
 from ._download import get_default_weights_path, download_weights_to
 
+# Note: Keras 3 has the KerasTensor class but not in Keras 2.
+KerasTensor = TypeVar("KerasTensor")
+
 
 def _batch_norm(name: str) -> layers.BatchNormalization:
     return layers.BatchNormalization(
@@ -23,11 +25,11 @@ def _batch_norm(name: str) -> layers.BatchNormalization:
 def _conv_block(
         stage: int,
         block: int,
-        inputs: tf.Tensor,
+        inputs: KerasTensor,
         nums_filters: Tuple[int, int, int],
         kernel_size: int = 3,
         stride: int = 2,
-) -> tf.Tensor:
+) -> KerasTensor:
     num_filters_1, num_filters_2, num_filters_3 = nums_filters
 
     conv_name_base = f"conv_stage{stage}_block{block}_branch"
@@ -77,16 +79,16 @@ def _conv_block(
 
     x = layers.Add()([x, shortcut])
 
-    return layers.Activation("relu", name=final_activation_name)(x)
+    return layers.Activation("relu", name=final_activation_name)(x)  # type: ignore
 
 
 def _identity_block(
         stage: int,
         block: int,
-        inputs: tf.Tensor,
+        inputs: KerasTensor,
         nums_filters: Tuple[int, int, int],
         kernel_size: int
-) -> tf.Tensor:
+) -> KerasTensor:
     num_filters_1, num_filters_2, num_filters_3 = nums_filters
 
     conv_name_base = f"conv_stage{stage}_block{block}_branch"
@@ -125,17 +127,18 @@ def _identity_block(
 
     x = layers.Add()([x, inputs])
 
-    return layers.Activation("relu", name=final_activation_name)(x)
+    return layers.Activation("relu", name=final_activation_name)(x)  # type: ignore
 
 
 def make_open_nsfw_model(
         input_shape: Tuple[int, int, int] = (224, 224, 3),
-        weights_path: Optional[str] = get_default_weights_path()
-) -> tf.keras.Model:
+        weights_path: Optional[str] = get_default_weights_path(),
+        name: str = "opennsfw2"
+) -> Model:
     image_input = layers.Input(shape=input_shape, name="input")
     x = image_input
 
-    x = tf.pad(x, [[0, 0], [3, 3], [3, 3], [0, 0]], "CONSTANT")
+    x = layers.ZeroPadding2D((3, 3))(x)
     x = layers.Conv2D(name="conv_1", filters=64, kernel_size=7, strides=2,
                       padding="valid")(x)
 
@@ -192,7 +195,7 @@ def make_open_nsfw_model(
     logits = layers.Dense(name="fc_nsfw", units=2)(x)
     output = layers.Activation("softmax", name="predictions")(logits)
 
-    model = tf.keras.Model(image_input, output)
+    model = Model(image_input, output, name=name)
 
     if weights_path is not None:
         if not os.path.isfile(weights_path):
